Package: gitcreds
Title: Query 'git' Credentials from 'R'
Version: 0.1.2
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("aut", "cre")),
    person("RStudio", role = c("cph", "fnd"))
  )
Description: Query, set, delete credentials from the 'git' credential
    store. Manage 'GitHub' tokens and other 'git' credentials. This
    package is to be used by other packages that need to authenticate to
    'GitHub' and/or other 'git' repositories.
License: MIT + file LICENSE
URL: https://gitcreds.r-lib.org/, https://github.com/r-lib/gitcreds
BugReports: https://github.com/r-lib/gitcreds/issues
Depends: R (>= 3.4)
Suggests: codetools, covr, knitr, mockery, oskeyring, rmarkdown,
        testthat (>= 3.0.0), withr
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Encoding: UTF-8
RoxygenNote: 7.2.1.9000
SystemRequirements: git
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2022-09-08 10:28:07 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre],
  RStudio [cph, fnd]
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Repository: CRAN
Date/Publication: 2022-09-08 10:42:55 UTC
