# Generated by roxygen2: do not edit by hand

S3method(format,gitcreds)
S3method(print,gitcreds)
export(gitcreds_approve)
export(gitcreds_cache_envvar)
export(gitcreds_delete)
export(gitcreds_fill)
export(gitcreds_get)
export(gitcreds_list)
export(gitcreds_list_helpers)
export(gitcreds_parse_output)
export(gitcreds_reject)
export(gitcreds_set)
