% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa-doc.R
\name{gitcreds_cache_envvar}
\alias{gitcreds_cache_envvar}
\title{Environment variable to cache the password for a URL}
\usage{
gitcreds_cache_envvar(url)
}
\arguments{
\item{url}{Character vector of URLs, they may contain user names
and paths as well. See details below.}
}
\value{
Character vector of environment variables.
}
\description{
\code{gitcreds_get()} caches credentials in environment variables.
\code{gitcreds_cache_envvar()} calculates the environment variaable name
that is used as the cache, for a URL.
}
\examples{
gitcreds_cache_envvar("https://github.com")
gitcreds_cache_envvar("https://api.github.com/path/to/endpoint")
gitcreds_cache_envvar("https://jane@github.com")
gitcreds_cache_envvar("https://another.site.github.com")
}
\seealso{
\code{\link[=gitcreds_get]{gitcreds_get()}}.
}
